import os
from datetime import timedelta
import signal
import time

from display_timedelta import display_timedelta

from .active_process_table import active_process_table
from .interruptable_runner import clean_table


def load_processes(ordering):
    with active_process_table() as t:
        clean_table(t)
        t = dict(t.items())
    return sorted(t.values(), key=lambda x: x[ordering])


def list_procesess(ordering):
    processes = load_processes(ordering)
    print(f"Active processes ({len(processes)}):")
    for proc in processes:
        print(render(proc))


def render(proc):
    return f"- {proc['name']} ({proc['pid']}) [started {display_timedelta(timedelta(seconds=time.time() - proc['timestamp']))} ago] {{{proc['cmd']}}}"


def stop_process(name):
    if len(name) == 0:
        print("No process name specified")
        return 1
    processes = load_processes("timestamp")
    with_name_prefix = [p for p in processes if p["name"].startswith(name)]
    if len(with_name_prefix) == 0:
        print(f"No process with name {name} found")
        return 1
    if len(with_name_prefix) > 1:
        print(f"Multiple processes with prefix {name} found:")
        for proc in with_name_prefix:
            print(render(proc))
        return 1
    proc = with_name_prefix[0]
    if proc["name"] != name:
        print(
            f"Process name {proc['name']} does not match {name}. Please type the full name."
        )
        return 1
    print(f"Stopping {render(proc)}")
    os.kill(proc["pid"], signal.SIGINT)
    return 0



import os

from appdirs import user_cache_dir
from permacache.locked_shelf import LockedShelf


def active_process_table():
    return LockedShelf(
        os.path.join(user_cache_dir("s31"), "active_process_table"),
        multiprocess_safe=True,
    )



import time
import uuid
import threading
import random

from .active_process_table import active_process_table


INTERRUPTED_BANNER = """
=======================================
INTERRUPTED BY USER
=======================================
"""

POLL_INTERVAL = 2


class InterruptableRunner:
    def __init__(self, name, pid, cmd_to_use):
        self.name = name
        self.pid = pid
        self.cmd_to_use = cmd_to_use
        self.guid = str(uuid.uuid4())
        self.timestamp = time.time()
        self.alive = False

    def pre(self):
        with active_process_table() as t:
            t[self.guid] = dict(
                name=self.name,
                pid=self.pid,
                cmd=self.cmd_to_use.cmd_line,
                timestamp=self.timestamp,
                last_update=self.timestamp,
            )
        self.alive = True
        while_alive_thread = threading.Thread(target=self.while_alive)
        while_alive_thread.start()

    def post(self):
        self.alive = False
        with active_process_table() as t:
            del t[self.guid]

    def while_alive(self):
        while self.alive:
            with active_process_table() as t:
                if self.guid in t:
                    val = t[self.guid]
                    val["last_update"] = time.time()
                    t[self.guid] = val
            time.sleep(random.uniform(0.5 * POLL_INTERVAL, 1.5 * POLL_INTERVAL))

    def run_checking_interrupt(self, fn, interrupted_banner_path=None):
        try:
            self.pre()
            exitcode = fn()
        except KeyboardInterrupt:
            exitcode = "interrupted"
            if interrupted_banner_path is not None:
                with open(interrupted_banner_path, "ab") as f:
                    f.write(INTERRUPTED_BANNER.encode("utf-8"))
        finally:
            self.post()
        return exitcode


def clean_table(apt):
    now = time.time()
    for guid, val in apt.items():
        if now - val["last_update"] > 2 * POLL_INTERVAL:
            del apt[guid]
